/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.client.render;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javax.vecmath.Point3f;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;

import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import buildcraft.lib.client.model.ModelUtil;
import buildcraft.lib.client.render.DetachedRenderer;

import buildcraft.builders.client.ClientArchitectTables;

@SideOnly(Side.CLIENT)
public enum RenderArchitectTables implements DetachedRenderer.IDetachedRenderer {
    INSTANCE;

    @Override
    public void render(EntityPlayer player, float partialTicks) {
        List<AxisAlignedBB> boxes = new ArrayList<>(ClientArchitectTables.BOXES.keySet());
        boxes.sort(
            Comparator.<AxisAlignedBB>comparingDouble(bb ->
                bb.func_189972_c().func_72438_d(player.func_174791_d())
            ).reversed()
        );
        List<BlockPos> poses = new ArrayList<>(ClientArchitectTables.SCANNED_BLOCKS.keySet());
        poses.sort(
            Comparator.<BlockPos>comparingDouble(pos ->
                new Vec3d(pos).func_72438_d(player.func_174791_d())
            ).reversed()
        );
        for (AxisAlignedBB bb : boxes) {
            GL11.glStencilMask(0xff);
            GL11.glClearStencil(1);
            GlStateManager.func_179086_m(GL11.GL_STENCIL_BUFFER_BIT);
            GL11.glEnable(GL11.GL_STENCIL_TEST);

            GL11.glStencilFunc(GL11.GL_ALWAYS, 1, 0xFF);
            GL11.glStencilOp(GL11.GL_ZERO, GL11.GL_ZERO, GL11.GL_REPLACE);
            GL11.glStencilMask(0xFF);
            GL11.glDepthMask(false);
            GL11.glColorMask(false, false, false, false);
            BufferBuilder buffer = Tessellator.func_178181_a().func_178180_c();
            buffer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181705_e);
            bb = bb.func_186662_g(0.01);
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181675_d();
            buffer.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181675_d();
            Tessellator.func_178181_a().func_78381_a();
            GL11.glStencilMask(0x00);
            GL11.glDepthMask(true);
            GL11.glColorMask(true, true, true, true);

            GlStateManager.func_179097_i();
            GL11.glStencilFunc(GL11.GL_EQUAL, 1, 0xFF);
            GlStateManager.func_179147_l();
            GlStateManager.func_187401_a(GlStateManager.SourceFactor.SRC_ALPHA, GlStateManager.DestFactor.ONE_MINUS_SRC_ALPHA);
            Minecraft.func_71410_x().field_71446_o.func_110577_a(
                new ResourceLocation(
                    "buildcraftbuilders",
                    "textures/blocks/scan.png"
                )
            );
            buffer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
            for (BlockPos pos : poses) {
                if (!bb.func_72326_a(new AxisAlignedBB(pos))) {
                    continue;
                }
                for (EnumFacing face : EnumFacing.field_82609_l) {
                    ModelUtil.createFace(
                        face,
                        new Point3f(pos.func_177958_n() + 0.5F, pos.func_177956_o() + 0.5F, pos.func_177952_p() + 0.5F),
                        new Point3f(0.5F, 0.5F, 0.5F),
                        new ModelUtil.UvFaceData(0, 0, 1, 1)
                    )
                        .lighti(15, 15)
                        .colouri(
                            255,
                            255,
                            255,
                            ClientArchitectTables.SCANNED_BLOCKS.get(pos)
                                * 50
                                / ClientArchitectTables.START_SCANNED_BLOCK_VALUE
                        )
                        .render(buffer);
                }
            }
            Tessellator.func_178181_a().func_78381_a();
            GlStateManager.func_179084_k();
            GlStateManager.func_179126_j();

            GL11.glDisable(GL11.GL_STENCIL_TEST);
        }
    }
}
